@@ -206,5 +206,6 @@ $AUTOMAKE --gnu;
 echo "Running autoconf${FORCE} ..."
 $AUTOCONF${FORCE}
 
-echo "You may now run \"./configure --enable-maintainer-mode --sysconfdir=/etc && make\"."
+echo "You may now run: 
+  ./configure --enable-maintainer-mode --sysconfdir=/etc && make"
 echo "(gcc users may want to add the option \"--enable-gcc-warnings\")"
