@@ -56,14 +56,22 @@
    slot.  Note that there may be several records with the same key.
 */
 
-
-
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdlib.h> 
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
-#include <sys/mman.h>
+#ifdef _WIN32
+# include <windows.h>
+#else
+# include <sys/mman.h>
+# ifndef MAP_FAILED
+#  define MAP_FAILED ((void*)-1)
+# endif
+#endif
 #include <sys/stat.h>
 #include "cdb.h"
 
@@ -105,6 +113,10 @@ cdb_init(struct cdb *cdbp, int fd)
 {
   struct stat st;
   unsigned char *mem;
+  unsigned fsize;
+#ifdef _WIN32
+  HANDLE hFile, hMapping;
+#endif
 
   /* get file size */
   if (fstat(fd, &st) < 0)
@@ -114,10 +126,23 @@ cdb_init(struct cdb *cdbp, int fd)
     errno = EPROTO;
     return -1;
   }
+  fsize = (unsigned)(st.st_size & 0xffffffffu);
   /* memory-map file */
-  if ((mem = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0)) ==
-      (unsigned char *)-1)
+#ifdef _WIN32
+  hFile = (HANDLE) _get_osfhandle(fd);
+  if (hFile == (HANDLE) -1)
     return -1;
+  hMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
+  if (!hMapping)
+    return -1;
+  mem = (unsigned char *)MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, 0);
+  if (!mem)
+    return -1;
+#else
+  mem = (unsigned char*)mmap(NULL, fsize, PROT_READ, MAP_SHARED, fd, 0);
+  if (mem == MAP_FAILED)
+    return -1;
+#endif /* _WIN32 */
 
   cdbp->cdb_fd = fd;
   cdbp->cdb_fsize = st.st_size;
@@ -148,7 +173,17 @@ void
 cdb_free(struct cdb *cdbp)
 {
   if (cdbp->cdb_mem) {
+#ifdef _WIN32
+    HANDLE hFile, hMapping;
+#endif
+#ifdef _WIN32
+    hFile = (HANDLE) _get_osfhandle(cdbp->cdb_fd);
+    hMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
+    UnmapViewOfFile((void*) cdbp->cdb_mem);
+    CloseHandle(hMapping);
+#else
     munmap((void*)cdbp->cdb_mem, cdbp->cdb_fsize);
+#endif /* _WIN32 */
     cdbp->cdb_mem = NULL;
   }
   cdbp->cdb_fsize = 0;
