@@ -348,7 +348,7 @@ put_cert (ksba_cert_t cert, int is_loaded, int is_trusted, void *fpr_buffer)
 
 
 /* Load certificates from the directory DIRNAME.  All certificates
-   matching the pattern "*.crt" are loaded.  We assume that
+   matching the pattern "*.crt" or "*.der"  are loaded.  We assume that
    certificates are DER encoded and not PEM encapsulated. The cache
    should be in a locked state when calling this fucntion.  */
 static gpg_error_t
@@ -379,8 +379,8 @@ load_certs_from_dir (const char *dirname, int are_trusted)
       if (*p == '.' || !*p)
         continue; /* Skip any hidden files and invalid entries.  */
       n = strlen (p);
-      if ( n < 5 || strcmp (p+n-4,".crt") )
-        continue; /* Not the desired "*.crt" pattern.  */
+      if ( n < 5 || (strcmp (p+n-4,".crt") && strcmp (p+n-4,".der")))
+        continue; /* Not the desired "*.crt" or "*.der" pattern.  */
       
       xfree (fname);
       fname = make_filename (dirname, p, NULL);
