@@ -1,6 +1,6 @@
 /* dirmngr.c - LDAP access
  *	Copyright (C) 2002 Klarälvdalens Datakonsult AB
- *      Copyright (C) 2003, 2004, 2006, 2007 g10 Code GmbH
+ *      Copyright (C) 2003, 2004, 2006, 2007, 2008 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -200,7 +200,7 @@ static ARGPARSE_OPTS opts[] = {
   "@\n(See the \"info\" manual for a complete listing of all commands and options)\n"
                     )},
 
-  {0}
+  { 0, NULL, 0, NULL }
 };
 
 #define DEFAULT_MAX_REPLIES 10
@@ -1208,7 +1208,7 @@ main (int argc, char **argv)
                                 "ldapservers.conf":"dirmngr_ldapservers.conf",
                                 NULL);
       filename_esc = percent_escape (filename, NULL);
-      printf ("ldapserverlist-file:%lu:\"%s\n", GC_OPT_FLAG_DEFAULT,
+      printf ("ldapserverlist-file:%lu:\"%s\n", flags | GC_OPT_FLAG_DEFAULT,
 	      filename_esc);
       xfree (filename_esc);
       xfree (filename);
