@@ -466,7 +466,7 @@ reader_callback (void *cb_value, char *buffer, size_t count,  size_t *nread)
   size_t nleft = count;
 
   /* FIXME: We might want to add some internal buffering because the
-     ksba code does not not any buffering for itself (because a ksba
+     ksba code does not do any buffering for itself (because a ksba
      reader may be detached from another stream to read other data and
      the it would be cumbersome to get back already buffered
      stuff).  */
@@ -590,7 +590,7 @@ ldap_wrapper (ctrl_t ctrl, ksba_reader_t *reader, const char *argv[])
      only viable solutions are either to have another thread
      responsible for logging the messages or to add an option to the
      wrapper modules to the logging on its own.  Given that we anyway
-     need a way to rip the child process and is best done using a
+     need a way to rip the child process and this best done using a
      general ripping thread, that thread can do the logging too. */
 
   *reader = NULL;
@@ -666,7 +666,8 @@ ldap_wrapper (ctrl_t ctrl, ksba_reader_t *reader, const char *argv[])
   ctx->next = wrapper_list;
   wrapper_list = ctx;
   if (opt.verbose)
-    log_info (_("ldap wrapper %d started\n"), (int)ctx->pid);
+    log_info ("ldap wrapper %d started (reader %p)\n",
+              (int)ctx->pid, ctx->reader);
 
   /* Need to wait for the first byte so we are able to detect an empty
      output and not let the consumer see an EOF without further error
@@ -696,7 +697,7 @@ ldap_wrapper (ctrl_t ctrl, ksba_reader_t *reader, const char *argv[])
 
 
 /* Perform an LDAP query.  Returns an gpg error code or 0 on success.
-   The function returns a new stream at R_FP. */
+   The function returns a new reader object at READER. */
 static gpg_error_t
 run_ldap_wrapper (ctrl_t ctrl, 
                   int ignore_timeout,
@@ -780,7 +781,7 @@ run_ldap_wrapper (ctrl_t ctrl,
 
 
 /* Perform a LDAP query using a given URL. On success a new ksba
-   reader is returned.  If HOST or PORT ar not 0, they are used to
+   reader is returned.  If HOST or PORT are not 0, they are used to
    override the values from the URL. */
 gpg_error_t
 url_fetch_ldap (ctrl_t ctrl, const char *url, const char *host, int port,
@@ -789,7 +790,7 @@ url_fetch_ldap (ctrl_t ctrl, const char *url, const char *host, int port,
   gpg_error_t err;
 
   err = run_ldap_wrapper (ctrl,
-                          1, /* Ignore explicit timeout because CRLS
+                          1, /* Ignore explicit timeout because CRLs
                                 might be very large. */
                           0,
                           opt.ldap_proxy,
