@@ -49,6 +49,32 @@
                                         gpg_error (GPG_ERR_ASS_PARAMETER), (t))
 
 
+
+/* Control structure per connection. */
+struct server_local_s 
+{
+  /* Data used to associate an Assuan context with local server data */
+  assuan_context_t assuan_ctx;
+
+  /* Per-session LDAP serfver.  */
+  ldap_server_t ldapservers;
+};
+
+
+
+
+/* Accessor for the local ldapservers variable. */
+ldap_server_t
+get_ldapservers_from_ctrl (ctrl_t ctrl)
+{
+  if (ctrl && ctrl->server_local)
+    return ctrl->server_local->ldapservers;
+  else
+    return NULL;
+}
+
+
+
 /* Copy the % and + escaped string S into the buffer D and replace the
    escape sequences.  Note, that it is sufficient to allocate the
    target string D as long as the source string S, i.e.: strlen(s)+1.
@@ -309,8 +335,43 @@ get_cert_local_ski (ctrl_t ctrl, const char *name, ksba_sexp_t keyid)
 }
 
 
+/* Ask the client via an inquiry to check the istrusted status of the
+   certificate specified by the hexified fingerprint HEXFPR.  Returns
+   0 if the certificate is trusted by the client or an error code.  */
+gpg_error_t
+get_istrusted_from_client (ctrl_t ctrl, const char *hexfpr)
+{
+  unsigned char *value;
+  size_t valuelen; 
+  int rc;
+  char request[100];
 
-/* Ask the client to return the certificate asscociated with the
+  if (!ctrl || !ctrl->server_local || !ctrl->server_local->assuan_ctx
+      || !hexfpr)
+    return gpg_error (GPG_ERR_INV_ARG);
+  
+  snprintf (request, sizeof request, "ISTRUSTED %s", hexfpr);
+  rc = assuan_inquire (ctrl->server_local->assuan_ctx, request,
+                       &value, &valuelen, 100);
+  if (rc)
+    {
+      log_error (_("assuan_inquire(%s) failed: %s\n"),
+                 request, gpg_strerror (rc));
+      return rc;
+    }
+  /* The expected data is: "1" or "1 cruft" (not a C-string).  */
+  if (valuelen && *value == '1' && (valuelen == 1 || spacep (value+1)))
+    rc = 0;
+  else
+    rc = gpg_error (GPG_ERR_NOT_TRUSTED);
+  xfree (value);
+  return rc;
+}
+
+
+
+
+/* Ask the client to return the certificate associated with the
    current command. This is sometimes needed because the client usually
    sends us just the cert ID, assuming that the request can be
    satisfied from the cache, where the cert ID is used as key. */
@@ -559,8 +620,8 @@ get_fingerprint_from_line (const char *line, unsigned char *fpr)
    Check whether the certificate with FINGERPRINT (SHA-1 hash of the
    entire X.509 certificate blob) is valid or not by consulting the
    CRL responsible for this certificate.  If the fingerprint has not
-   been given or the certificate is not know, the function 
-   inquires the certficate using the
+   been given or the certificate is not known, the function 
+   inquires the certificate using an
 
       INQUIRE TARGETCERT
 
@@ -640,7 +701,7 @@ cmd_checkcrl (assuan_context_t ctx, char *line)
    been done for this certificate recently (we always cache OCSP
    responses for a couple of minutes). If the fingerprint has not been
    given or there is no cached result, the function inquires the
-   certificate using the
+   certificate using an
 
       INQUIRE TARGETCERT
 
@@ -1120,7 +1181,7 @@ cmd_cachecert (assuan_context_t ctx, char *line)
 
 /* VALIDATE 
 
-   Validate a certificate using the certificate validation fucntion
+   Validate a certificate using the certificate validation function
    used internally by dirmngr.  This command is only useful for
    debugging.  To get the actual certificate, this command immediately
    inquires it using
@@ -1177,7 +1238,7 @@ cmd_validate (assuan_context_t ctx, char *line)
       }
   }
 
-  err = validate_cert_chain (ctrl, cert, NULL, VALIDATE_MODE_CERT);
+  err = validate_cert_chain (ctrl, cert, NULL, VALIDATE_MODE_CERT, NULL);
 
  leave:
   if (err)
@@ -1207,7 +1268,7 @@ register_commands (assuan_context_t ctx)
     { "VALIDATE",   cmd_validate },
     { "INPUT",      NULL },
     { "OUTPUT",     NULL },
-    { NULL }
+    { NULL, NULL }
   };
   int i, j, rc;
 
