@@ -1,5 +1,5 @@
 /* util.h
- *	Copyright (C) 2004 g10 Code GmbH
+ *	Copyright (C) 2004, 2008 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef UTIL_H
@@ -35,7 +34,7 @@
 #include "../jnlib/dotlock.h"
 
 
-/*-- b64enc.c --*/
+/*-- b64enc.c and b64dec.c --*/
 struct b64state 
 { 
   unsigned int flags;
@@ -44,12 +43,19 @@ struct b64state
   FILE *fp;
   char *title;
   unsigned char radbuf[4];
+  int stop_seen:1;
+  int invalid_encoding:1;
 };
 gpg_error_t b64enc_start (struct b64state *state, FILE *fp, const char *title);
 gpg_error_t b64enc_write (struct b64state *state,
                           const void *buffer, size_t nbytes);
 gpg_error_t b64enc_finish (struct b64state *state);
 
+gpg_error_t b64dec_start (struct b64state *state, const char *title);
+gpg_error_t b64dec_proc (struct b64state *state, void *buffer, size_t length,
+                         size_t *r_nbytes);
+gpg_error_t b64dec_finish (struct b64state *state);
+
 
 
 
