@@ -338,10 +338,16 @@ is_root_cert (ksba_cert_t cert, const char *issuerdn, const char *subjectdn)
 
 /* Validate the certificate CHAIN up to the trust anchor. Optionally
    return the closest expiration time in R_EXPTIME (this is useful for
-   caching issues).  MODE is one of the VALIDATE_MODE_* constants. */
+   caching issues).  MODE is one of the VALIDATE_MODE_* constants. 
+
+   If R_TRUST_ANCHOR is not NULL and the validation would fail only
+   because the root certificate is not trusted, the hexified
+   fingerprint of that root certificate is stored at R_TRUST_ANCHOR
+   and success is returned.  The caller needs to free the value at
+   R_TRUST_ANCHOR; in all other cases NULL is stored there.  */
 gpg_error_t
 validate_cert_chain (ctrl_t ctrl, ksba_cert_t cert, ksba_isotime_t r_exptime,
-                     int mode)
+                     int mode, char **r_trust_anchor)
 {
   gpg_error_t err = 0;
   int depth, maxdepth;
@@ -359,6 +365,9 @@ validate_cert_chain (ctrl_t ctrl, ksba_cert_t cert, ksba_isotime_t r_exptime,
     *r_exptime = 0;
   *exptime = 0;
 
+  if (r_trust_anchor)
+    *r_trust_anchor = NULL;
+
   if (!opt.system_daemon)
     {
       /* For backward compatibility we only do this in daemon mode.  */
@@ -527,8 +536,15 @@ validate_cert_chain (ctrl_t ctrl, ksba_cert_t cert, ksba_isotime_t r_exptime,
               log_error (_("root certificate is not marked trusted"));
               fpr = get_fingerprint_hexstring (subject_cert);
               log_info (_("fingerprint=%s\n"), fpr? fpr : "?");
-              xfree (fpr);
               dump_cert ("issuer", subject_cert);
+              if (r_trust_anchor)
+                {
+                  /* Caller wants to do another trustiness check.  */
+                  *r_trust_anchor = fpr;
+                  err = 0;
+                }
+              else
+                xfree (fpr);
             }
           else 
             {
@@ -557,7 +573,12 @@ validate_cert_chain (ctrl_t ctrl, ksba_cert_t cert, ksba_isotime_t r_exptime,
           }
 
           if (opt.verbose)
-            log_info ("root certificate is good\n");
+            {
+              if (r_trust_anchor && *r_trust_anchor)
+                log_info ("root certificate is good but not trusted\n");
+              else
+                log_info ("root certificate is good and trusted\n");
+            }
 
           break;  /* Okay: a self-signed certicate is an end-point. */
         }
@@ -728,16 +749,25 @@ validate_cert_chain (ctrl_t ctrl, ksba_cert_t cert, ksba_isotime_t r_exptime,
     }
 
   if (!err && opt.verbose)
-    log_info ("target certificate is valid\n");
+    {
+      if (r_trust_anchor && *r_trust_anchor)
+        log_info ("target certificate may be valid\n");
+      else
+        log_info ("target certificate is valid\n");
+    }
   else if (err && opt.verbose)
     log_info ("target certificate is NOT valid\n");
 
   
  leave:
-  if (!err)
+  if (!err && !(r_trust_anchor && *r_trust_anchor))
     {
       /* With no error we can update the validation cache.  We do this
-         for all certificates in the chain. */
+         for all certificates in the chain.  Note that we can't use
+         the cache if the caller requested to check the trustiness of
+         the root certificate himself.  Adding such a feature would
+         require us to also store the fingerprint of root
+         certificate.  */
       chain_item_t citem;
       time_t validated_at = get_time ();
 
@@ -769,6 +799,11 @@ validate_cert_chain (ctrl_t ctrl, ksba_cert_t cert, ksba_isotime_t r_exptime,
       xfree (chain);
       chain = ci_next;
     }
+  if (err && r_trust_anchor && *r_trust_anchor)
+    {
+      xfree (*r_trust_anchor);
+      *r_trust_anchor = NULL;
+    }
   return err;
 }
 
@@ -833,11 +868,7 @@ check_cert_sig (ksba_cert_t issuer_cert, ksba_cert_t cert)
       log_error (_("unknown hash algorithm `%s'\n"), algoid? algoid:"?");
       return gpg_error (GPG_ERR_GENERAL);
     }
-  /* At some point in time a bug slipped into Libgcrypt returning
-     "ripemd160" as canonical name but only accepting "rmd160" in
-     pk_verify.  This will be fixed in libgcrypt 1.2.4 but we use a
-     workaround here. */
-  s = (algo == GCRY_MD_RMD160)? "rmd160": gcry_md_algo_name (algo);
+  s = gcry_md_algo_name (algo);
   for (i=0; *s && i < sizeof algo_name - 1; s++, i++)
     algo_name[i] = tolower (*s);
   algo_name[i] = 0;   
@@ -849,13 +880,8 @@ check_cert_sig (ksba_cert_t issuer_cert, ksba_cert_t cert)
       return err;
     }
   if (DBG_HASHING)
-    {
-#ifdef HAVE_GCRY_MD_DEBUG
-      gcry_md_debug (md, "hash.cert");
-#else
-      gcry_md_start_debug (md, "hash.cert");
-#endif
-    }
+    gcry_md_debug (md, "hash.cert");
+
   err = ksba_cert_hash (cert, 1, HASH_FNC, md);
   if (err)
     {
