@@ -1,3 +1,6 @@
+#include <config.h>
+#undef _ASSUAN_ONLY_GPG_ERRORS
+
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
@@ -8,7 +11,7 @@
 #include <gcrypt.h>
 
 #define FNAME "testcert.der"
-static char* fname;
+static const char* fname;
 
 assuan_context_t entry_ctx;
 
@@ -169,9 +172,12 @@ main( int argc, char** argv )
   int err;
 
   rc = start_dirmngr();
-  if( rc ) print_assuan_error(rc);
-  if( argc > 1 ) fname = argv[1];
-  else fname = FNAME;
+  if( rc )
+    print_assuan_error(rc);
+  if( argc > 1 )
+    fname = argv[1];
+  else
+    fname = FNAME;
 
   fp = fopen ( fname, "r");
   if (!fp) {
